//=============================================================================
// KillObject
//
// This is mostly an information object.
// It determines if an object should be destroyed or not.
// The routine to identify the "unidentifyable" objects tries to id the sought
// class by determining the class name, then trying to load the class
// dynamically. This will fail if the classname is unwritable, indicating the
// "unidentifyable" class has been found. If the dynamic loading fails, it is
// a regular class.
//
// Author: }TCP{Wolf
//   Date: February 2012
//=============================================================================

class KillObject extends Info;

// RUNTIME
var String PackageName;    // for unidentifyable classnames, the package must be given at least
var Class<Actor> actualClass; // contains the actual class that is going to be destroyed, if "none" the class has not been identified yet
var KillObject next;       // chain link
var POCKiller mut;         // reference to mutator so we can access config

//=============================================================================
// MANAGEMENT
//=============================================================================

// Add a new object to the link chain
function AddKillObject(KillObject newOne)
{
	if (next == none)
		next = NewOne;
	else
		next.AddKillObject(newOne);
}


// Call this on ROOT, then the whole chained list will be printed
function ListKillObjects(int counter)
{
	if (actualClass == none)
		l(counter$": Class="$PackageName$".*Classname not yet identified*");
	else
		l(counter$": Class="$actualClass);
	if (next != none)
		next.ListKillObjects(counter+1);
}


// Check if THIS is an object to be killed
// isRelevant is adapted from mutators, so "false" means "kill it", "true" means "keep it"
function bool isRelevant(Actor Other)
{
	local String s, fname, cname;
	local int pos;
	//local Class<Actor> tempLoadClass;

	fname = ""$Other.Class;

	// check own entry
	if (actualClass == none) // this means we have not identified the object yet...
	{
		// check only if same package
		pos=Instr(fname,".");
		s=left(fname,pos);	// s now contains the package name
		if (s ~= PackageName)	// yea that's us...
		{
			cname = right(fname, len(fname)-len(PackageName)-1);
			if (mut.bDebug) l("Looking at "$s$"."$cname);
			/* ***
			//  I have taken this routing out again, not sure there could be any class created for which loading would fail,
			//  considering that even obfuscated classes load by name... whyever that is... :-S
			tempLoadClass = Class<Actor>(DynamicLoadObject(s$"."$cname, class'Class'));
			if (mut.bDebug) l("Loaded class attempt result = "$tempLoadClass);
			if (tempLoadClass == none)	// failed! so THIS IS IT!
			{
				actualClass = Other.Class;
				if (mut.bDebug) l("Assuming we found the unidentifyable class by failure to load....");
			}*/
			/*  *******************************************************************************
			// NOTE THIS detection sequence does NOT work, despite the debug output telling me
			// that there are different names!
			else if(fname != (""$tempLoadClass))
			{
				actualClass = Other.Class;
				if (mut.bDebug) l("Assuming we found the unidentifyable class by static/dynamic name discrepancy....");
			}
			// so... we are going to use the CleanASCII string function... a deriviate of
			// multiple other mods I have used similar functions in to clean up "bad" input...
			*******************************************************************************  */
			/*else*/
			// this is the only somewhat reliable method I figured to access obfuscated classes
			if( (""$cname) != CleanASCII(""$cname))
			{
				actualClass = Other.Class;
				if (mut.bDebug) l("Assuming we found the unidentifyable class by bad name....");
			}
			else
				if (mut.bDebug) l("Object does not appear to be of interest...");				
		}
	}

	if (actualClass != none)	// if it was none before, it MAY have been loaded by now...
	{
		// see if classes match
		if (actualClass == Other.class)
		{
			if (mut.bDebug)
			{
				if (mut.bSelective)
					l("Class "$Other.class$" matches "$actualClass$" -> object is to be investigated...");
				else
					l("Class "$Other.class$" matches "$actualClass$" -> object is to be destroyed...");
			}
			return false;	// yup... found it... not relevant
		}
		else
			if (mut.bDebug) l("Class "$Other.class$" does not match "$actualClass);
	}

	// we're done checking... next in line...
	if (next != none)
		return next.isRelevant(Other);

	if (mut.bDebug) l("End of check chain, object is still relevant!");
	return true; // default = KEEP !
}


// A function to clan possible classnames
static function String CleanASCII(string what)
{
	local String Result, char;

	char=left(what,1);
	while(char!="")
	{
		if (    ( (asc(char)>64) && (asc(char)<91) )    ||   // A-Z
						( (asc(char)>96) && (asc(char)<123) )   ||   // a-z
						( (asc(char)>47) && (asc(char)<58) )    ||   // 0-9
						( asc(char) == 95 )                          // The underscore _
				)
			Result=Result$char;
		what=right(what, len(what)-1);
		char=left(what,1);
	}

	return Result;
}


//=============================================================================
// STUPID METHODS
//=============================================================================

// Simply log out stuff with a prefix...
function l (String what)
{
	log(what,'POCKiller');
}



defaultproperties
{
	bHiddenEd=True
	DrawType=DT_None
	Style=STY_None
	Texture=None
	Fatness=0
}
