//=============================================================================
// POCDelayedDestroy
//
// A cheap class that executes its main code in "timer()".
//
// What it does is simply check a previously detected actor if it should be
// destroyed or not.
//
// Author: }TCP{Wolf
//   Date: February 2012
//=============================================================================

class POCDelayedDestroy extends Info;

// RUNTIME
var float theTimer;  // is set by caller from outside before spawn!
var POCKiller mut;   // a reference variable to the calling mutator, so we can access all of its vars
var int runcount;    // well... what does that look like it does...?

// the only reference NOT set by default before spawn...
var Actor suspiciousObject;


function PostBeginPlay()
{
	// sanity check
	if ((theTimer == 0.0) || (mut == none))
	{
		log("INIT ERROR for delayed POCKiller :( Something very bad is going on...",'FATAL');
		destroy();
	}

	SetTimer(theTimer, True);
}


function Timer()
{
	local Actor trigger;
	local Pawn p;

	runcount++;
	trigger = none;

	if (mut.bDebug) mut.l("Delayed POC Killer runcount "$runcount$" for object: "$suspiciousObject);

	// let's check the object...
	if (suspiciousObject == none)
	{
		mut.l("Some wizeguy already removed the object I was supposed to look out for... hmmm...");
		Destroy();
		return; // <-- the esoterics among you please ask yourselves if this return will ever execute and why, if not, I still put it there... :)
	}

	// righto... object still exists, owner...?
	if ((suspiciousObject.Owner == none) && (suspiciousObject.instigator == none))
	{
		mut.l("Woah epic fail, suspicious objet has no owner and no instigator!");
		Destroy();
		return;
	}

	if (mut.isTriggerObject(suspiciousObject.Owner))
		trigger = suspiciousObject.Owner;
	else if (mut.isTriggerObject(suspiciousObject.Instigator))
		trigger = suspiciousObject.Instigator;

	if (trigger == none)
	{
		mut.l("Neither Owner nor Instigator of object "$suspiciousObject$" is a trigger");
		Destroy();
		return;
	}

	p = Pawn(trigger);
	if (p.PlayerReplicationInfo == none)
	{
		if (mut.bDebug) mut.l("No PRI (yet) on "$p);
	}
	else
	{
		// w00t We have a PlayerReplicationInfo...
		// still... it could be "blank" yet... need to check for that too
		if ((runcount == 1) && (p.PlayerReplicationInfo.PlayerName ~= ""))
		{
			if (mut.bDebug)
				mut.l("Firstrun encountered blank playername, skip this time...");
		}
		else	// assume name is correct, even if blank!
		{
			if (mut.playerNameMatch(p.PlayerReplicationInfo.PlayerName))
			{
				// Player Name match
				mut.l("Destroying "$suspiciousObject$" belonging to "$p.PlayerReplicationInfo.PlayerName);
				suspiciousObject.Destroy();
			}
			else
			{
				if (mut.bDebug)
					mut.l("No player name match. "$suspiciousObject$" not destroyed.");
			}
			// cleanup...
			suspiciousObject = none;
			Destroy();
			// DONE YAY
			return;
		}
	}

	// one run through... another?
	if (runcount >= mut.SelectiveDestructionAttempts)	// no, we're through...
	{
		if (mut.bDebug) mut.l(self$" dying... object "$suspiciousObject$" not destroyed.");
		Destroy();
		return;
	}
}




defaultproperties
{
	runcount=0

	bHiddenEd=True
	DrawType=DT_None
	Style=STY_None
	Texture=None
	Fatness=0
}
